/*
 * Jalview - A Sequence Alignment Editor and Viewer (Version 2.9.0b2)
 * Copyright (C) 2015 The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.ext.rbvi.chimera;

import static org.testng.AssertJUnit.assertEquals;
import static org.testng.AssertJUnit.assertTrue;

import java.awt.Color;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.testng.annotations.Test;

public class ChimeraCommandsTest
{
  @Test(groups = { "Functional" })
  public void testAddColourRange()
  {
    Map<Color, Map<Integer, Map<String, List<int[]>>>> map = new LinkedHashMap<Color, Map<Integer, Map<String, List<int[]>>>>();
    ChimeraCommands.addColourRange(map, Color.pink, 1, 2, 4, "A");
    ChimeraCommands.addColourRange(map, Color.pink, 1, 8, 8, "A");
    ChimeraCommands.addColourRange(map, Color.pink, 1, 5, 7, "B");
    ChimeraCommands.addColourRange(map, Color.red, 1, 3, 5, "A");
    ChimeraCommands.addColourRange(map, Color.red, 0, 1, 4, "B");
    ChimeraCommands.addColourRange(map, Color.orange, 0, 5, 9, "C");

    // three colours mapped
    assertEquals(3, map.keySet().size());

    // Red has two models, Pink and Orange one each
    assertEquals(2, map.get(Color.red).keySet().size());
    assertEquals(1, map.get(Color.orange).keySet().size());
    assertEquals(1, map.get(Color.pink).keySet().size());

    // pink model 1 has two chains, red.0 / red.1 / orange.0 one each
    assertEquals(2, map.get(Color.pink).get(1).keySet().size());
    assertEquals(1, map.get(Color.red).get(0).keySet().size());
    assertEquals(1, map.get(Color.red).get(1).keySet().size());
    assertEquals(1, map.get(Color.orange).get(0).keySet().size());

    // inspect positions
    List<int[]> posList = map.get(Color.pink).get(1).get("A");
    assertEquals(2, posList.size());
    assertTrue(Arrays.equals(new int[] { 2, 4 }, posList.get(0)));
    assertTrue(Arrays.equals(new int[] { 8, 8 }, posList.get(1)));

    posList = map.get(Color.pink).get(1).get("B");
    assertEquals(1, posList.size());
    assertTrue(Arrays.equals(new int[] { 5, 7 }, posList.get(0)));

    posList = map.get(Color.red).get(0).get("B");
    assertEquals(1, posList.size());
    assertTrue(Arrays.equals(new int[] { 1, 4 }, posList.get(0)));

    posList = map.get(Color.red).get(1).get("A");
    assertEquals(1, posList.size());
    assertTrue(Arrays.equals(new int[] { 3, 5 }, posList.get(0)));

    posList = map.get(Color.orange).get(0).get("C");
    assertEquals(1, posList.size());
    assertTrue(Arrays.equals(new int[] { 5, 9 }, posList.get(0)));
  }

  @Test(groups = { "Functional" })
  public void testBuildColourCommands()
  {

    Map<Color, Map<Integer, Map<String, List<int[]>>>> map = new LinkedHashMap<Color, Map<Integer, Map<String, List<int[]>>>>();
    ChimeraCommands.addColourRange(map, Color.blue, 0, 2, 5, "A");
    ChimeraCommands.addColourRange(map, Color.blue, 0, 7, 7, "B");
    ChimeraCommands.addColourRange(map, Color.blue, 0, 9, 23, "A");
    ChimeraCommands.addColourRange(map, Color.blue, 1, 1, 1, "A");
    ChimeraCommands.addColourRange(map, Color.blue, 1, 4, 7, "B");
    ChimeraCommands.addColourRange(map, Color.yellow, 1, 8, 8, "A");
    ChimeraCommands.addColourRange(map, Color.yellow, 1, 3, 5, "A");
    ChimeraCommands.addColourRange(map, Color.red, 0, 3, 5, "A");

    // Colours should appear in the Chimera command in the order in which
    // they were added; within colour, by model, by chain, and positions as
    // added
    String command = ChimeraCommands.buildColourCommands(map).get(0);
    assertEquals(
            "color #0000ff #0:2-5.A,9-23.A,7.B|#1:1.A,4-7.B; color #ffff00 #1:8.A,3-5.A; color #ff0000 #0:3-5.A",
            command);
  }
}
